#ifndef MOTOR_CTRL_H_
#define MOTOR_CTRL_H_

#if (ARDUINO >= 100)
 #include <Arduino.h>
#else
 #include <WProgram.h>
 #include <pins_arduino.h>
#endif

// Needs pin change interrupt
#include "PinChangeInterrupt.h"

// PID control
#include "PID_v1.h"

// Manually tuning for now
// #include "PID_AutoTune_v0.h"

#define MTR_CTRL_POS 0
#define MTR_CTRL_SPD 1

class motor_ctrl {
  public:
    // pin1, pin2 needs to be PWM enabled
    // sensorPin1, sensorPin2 needs to be PCINT enabled
    motor_ctrl( int8_t pin1, int8_t pin2 , int8_t sensorPin1, int8_t sensorPin2 );
    ~motor_ctrl();

    void clear_pos();

    // There's an init cuz it needs to be done in setup
    void init();

    int8_t get_sensorPin( int8_t id );

    int32_t get_currentPos();
    int32_t get_setPos();
    
    // Register directly to PCINT
    void togglePin0(void);
    void togglePin1(void);

    // route PID controller parameter out
    void SetSampleTime(int st);
    void SetTunings(double Kp, double Ki, double Kd);
    void SetMode(int Mode);
    void SetControllerDirection(int Dir);

    // Calc Difference
    void process_motor(void);

    void set_point(double setpoint);  // to reduce double drifting, use long position
    void set_position( unsigned long setpos );

    boolean moved();

    // Fuzzy stop
    void stop_motor();
  protected:
    motor_ctrl();

    PID *m_PID_ctrl;

    int32_t m_mtr_pos;

    int8_t m_Pins[2];       // output pins
    int8_t m_SensorPins[2]; // input pins

    // Static member, every motor is the same to add or subtract mtr position
    const static int8_t ms_trans_matrix[4][4];

    // The encoder status
    uint8_t m_sensor_val;
    uint8_t m_sensor_val_last;

    // Function to modify motor position
    void _sense_motor_pos_();
    // Set the output of motor
    void _set_motor_output_();

    // Motor Value (for now let's do it manually, if the PID doesn't work, call inverse)
    double m_motor_out_val;  // -255 (reverse) to 255 (positive)

    // Motor Mode
    //  1: Speed
    //  0: Position (Default)
    int8_t m_mode;

    // Double value for input (needs to be calculated, or give speed difference)
    double m_motor_diff;

    // Set value
    double m_set_point;
    int32_t m_set_pos;

    // Changed
    boolean m_moved;
};

#endif  //MOTOR_CTRL_H_
